package de.kif.backend.route

import de.kif.backend.authenticate
import de.kif.backend.authenticateOrRedirect
import de.kif.backend.backup.Backup
import de.kif.backend.route.api.error
import de.kif.backend.view.MainTemplate
import de.kif.backend.view.MenuTemplate
import de.kif.common.RepositoryType
import de.kif.common.model.Permission
import io.ktor.application.call
import io.ktor.html.respondHtmlTemplate
import io.ktor.http.ContentType
import io.ktor.http.HttpStatusCode
import io.ktor.http.content.PartData
import io.ktor.http.content.forEachPart
import io.ktor.http.content.streamProvider
import io.ktor.request.receiveMultipart
import io.ktor.response.respondRedirect
import io.ktor.response.respondText
import io.ktor.routing.Route
import io.ktor.routing.get
import io.ktor.routing.post
import kotlinx.html.*

fun Route.account() {
    get("/account") {
        authenticateOrRedirect { user ->
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.ACCOUNT
                }
                content {
                    h1 { +"Account" }
                    div {
                        +"Welcome ${user.username}!"
                        br {}
                        +"You have the following rights: ${user.permissions}"
                        br {}
                        a("/logout") {
                            button(classes = "form-btn") {
                                +"Logout"
                            }
                        }
                    }

                    div {
                        if (user.checkPermission(Permission.ROOM)) {
                            a("/account/backup/rooms.json", classes = "form-btn") {
                                attributes["download"] = "rooms-backup"
                                +"Create room backup"
                            }
                        }
                        if (user.checkPermission(Permission.USER)) {
                            a("/account/backup/users.json", classes = "form-btn") {
                                attributes["download"] = "users-backup"
                                +"Create user backup"
                            }
                        }
                        if (user.checkPermission(Permission.POST)) {
                            a("/account/backup/posts.json", classes = "form-btn") {
                                attributes["download"] = "posts-backup"
                                +"Create post backup"
                            }
                        }
                        if (user.checkPermission(Permission.WORK_GROUP)) {
                            a("/account/backup/work-groups.json", classes = "form-btn") {
                                attributes["download"] = "work-groups-backup"
                                +"Create work group backup"
                            }
                        }
                        if (
                            user.checkPermission(Permission.WORK_GROUP) &&
                            user.checkPermission(Permission.ROOM) &&
                            user.checkPermission(Permission.SCHEDULE)
                        ) {
                            a("/account/backup/schedules.json", classes = "form-btn") {
                                attributes["download"] = "schedules-backup"
                                +"Create schedule backup"
                            }
                        }

                        if (user.checkPermission(Permission.ADMIN)) {
                            a("/account/backup.json", classes = "form-btn") {
                                attributes["download"] = "backup.json"
                                +"Create backup"
                            }
                        }
                    }
                    div {
                        form(
                            action = "/account/import",
                            method = FormMethod.post,
                            encType = FormEncType.multipartFormData
                        ) {
                            div("form-group") {
                                label {
                                    htmlFor = "backup"
                                    +"Backup image"
                                }
                                input(
                                    name = "backup",
                                    classes = "form-btn",
                                    type = InputType.file
                                ) {
                                    id = "backup"
                                    value = "Select backup image"
                                    accept = ".json"
                                }
                            }

                            div("form-switch-group") {
                                div("form-group form-switch") {
                                    input(
                                        name = "reset",
                                        classes = "form-control",
                                        type = InputType.checkBox
                                    ) {
                                        id = "reset"
                                        checked = false
                                    }
                                    label {
                                        htmlFor = "reset"
                                        +"Reset"
                                    }
                                }
                            }

                            div("form-group") {
                                button(type = ButtonType.submit, classes = "form-btn btn-primary") {
                                    +"Import"
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    get("/account/backup/rooms.json") {
        authenticate(Permission.ROOM) {
            call.respondText(Backup.backup(RepositoryType.ROOM), ContentType.Application.Json, HttpStatusCode.OK)
        } onFailure {
            call.error(HttpStatusCode.Unauthorized)
        }
    }

    get("/account/backup/users.json") {
        authenticate(Permission.USER) {
            call.respondText(Backup.backup(RepositoryType.USER), ContentType.Application.Json, HttpStatusCode.OK)
        } onFailure {
            call.error(HttpStatusCode.Unauthorized)
        }
    }

    get("/account/backup/posts.json") {
        authenticate(Permission.POST) {
            call.respondText(Backup.backup(RepositoryType.POST), ContentType.Application.Json, HttpStatusCode.OK)
        } onFailure {
            call.error(HttpStatusCode.Unauthorized)
        }
    }

    get("/account/backup/work-groups.json") {
        authenticate(Permission.WORK_GROUP) {
            call.respondText(Backup.backup(RepositoryType.WORK_GROUP), ContentType.Application.Json, HttpStatusCode.OK)
        } onFailure {
            call.error(HttpStatusCode.Unauthorized)
        }
    }

    get("/account/backup/schedules.json") {
        authenticate(Permission.ROOM, Permission.WORK_GROUP, Permission.SCHEDULE) {
            call.respondText(Backup.backup(RepositoryType.SCHEDULE), ContentType.Application.Json, HttpStatusCode.OK)
        } onFailure {
            call.error(HttpStatusCode.Unauthorized)
        }
    }

    get("/account/backup.json") {
        authenticate(Permission.ADMIN) {
            call.respondText(Backup.backup(*RepositoryType.values()), ContentType.Application.Json, HttpStatusCode.OK)
        } onFailure {
            call.error(HttpStatusCode.Unauthorized)
        }
    }

    post("/account/import") {
        authenticateOrRedirect(Permission.ADMIN) {
            var reset = false
            var import = ""

            call.receiveMultipart().forEachPart { part ->
                val name = part.name ?: return@forEachPart
                when (part) {
                    is PartData.FormItem -> {
                        if (name == "reset" && part.value == "on") {
                            reset = true
                        }
                    }
                    is PartData.FileItem -> {
                        import = part.streamProvider().bufferedReader().readText()
                    }
                }
            }

            if (reset) {
                Backup.restore(import)
            } else {
                Backup.import(import)
            }

            call.respondRedirect("/account")
        }
    }
}
