package de.kif.backend.route

import de.kif.backend.authenticateOrRedirect
import de.kif.backend.isAuthenticated
import de.kif.backend.repository.RoomRepository
import de.kif.backend.repository.ScheduleRepository
import de.kif.backend.repository.WorkGroupRepository
import de.kif.backend.view.MainTemplate
import de.kif.backend.view.MenuTemplate
import de.kif.backend.view.TableTemplate
import de.kif.common.CALENDAR_GRID_WIDTH
import de.kif.common.Search
import de.kif.common.model.Permission
import de.kif.common.model.Room
import de.kif.common.model.Schedule
import io.ktor.application.call
import io.ktor.html.insert
import io.ktor.html.respondHtmlTemplate
import io.ktor.response.respondRedirect
import io.ktor.routing.Route
import io.ktor.routing.get
import kotlinx.css.CSSBuilder
import kotlinx.css.Color
import kotlinx.css.pct
import kotlinx.css.rem
import kotlinx.html.*
import kotlin.collections.component1
import kotlin.collections.component2
import kotlin.collections.set
import kotlin.math.max
import kotlin.math.min

const val MINUTES_OF_DAY = 24 * 60

private fun DIV.calendarCell(schedule: Schedule?) {
    if (schedule != null) {
        span("calendar-entry") {
            attributes["style"] = CSSBuilder().apply {
                val size = schedule.workGroup.length / CALENDAR_GRID_WIDTH.toDouble()
                val pos = (schedule.time % CALENDAR_GRID_WIDTH) / CALENDAR_GRID_WIDTH.toDouble()

                this.left = (pos * 100).pct
                this.top = (pos * 100).pct + 0.1.rem

                this.width = (size * 100).pct
                this.height = (size * 100).pct - 0.2.rem

                val c = schedule.workGroup.track?.color
                if (c != null) {
                    backgroundColor = Color(c.toString())
                    color = Color(c.calcTextColor().toString())
                }
            }.toString()
            attributes["data-language"] = schedule.workGroup.language.code
            attributes["data-id"] = schedule.id.toString()

            +schedule.workGroup.name

            div("calendar-tools") {
                a(
                    classes = "calendar-tools-m10",
                    href = "/calendar/${schedule.day}/${schedule.id}/delete?redirect=/calendar/${schedule.day}/${schedule.room.id}/${schedule.time - 10}/${schedule.workGroup.id}"
                ) { +"-10" }
                a(
                    classes = "calendar-tools-m5",
                    href = "/calendar/${schedule.day}/${schedule.id}/delete?redirect=/calendar/${schedule.day}/${schedule.room.id}/${schedule.time - 5}/${schedule.workGroup.id}"
                ) { +"-05" }
                a(
                    classes = "calendar-tools-reset",
                    href = "/calendar/${schedule.day}/${schedule.id}/delete?redirect=/calendar/${schedule.day}/${schedule.room.id}/${schedule.time / CALENDAR_GRID_WIDTH * CALENDAR_GRID_WIDTH}/${schedule.workGroup.id}"
                ) { +"reset" }
                a(
                    classes = "calendar-tools-p5",
                    href = "/calendar/${schedule.day}/${schedule.id}/delete?redirect=/calendar/${schedule.day}/${schedule.room.id}/${schedule.time + 5}/${schedule.workGroup.id}"
                ) { +"+05" }
                a(
                    classes = "calendar-tools-p10",
                    href = "/calendar/${schedule.day}/${schedule.id}/delete?redirect=/calendar/${schedule.day}/${schedule.room.id}/${schedule.time + 10}/${schedule.workGroup.id}"
                ) { +"+10" }
                a(
                    classes = "calendar-tools-del",
                    href = "/calendar/${schedule.day}/${schedule.id}/delete"
                ) { +"del" }
            }
        }
    }
}

private fun DIV.renderTimeToRoom(
    day: Int,
    from: Int,
    to: Int,
    rooms: List<Room>,
    schedules: Map<Room, Map<Int, Schedule>>,
    allowEdit: Boolean
) {
    val gridLabelWidth = 60
    val minutesOfDay = to - from

    div("calendar-table-time-to-room") {
        div("calendar-header") {
            div("calendar-cell") {
                span {
                    +"Room"
                }
            }

            for (i in 0 until minutesOfDay / gridLabelWidth) {
                div("calendar-cell") {
                    span {
                        val time = ((i * gridLabelWidth + from) % MINUTES_OF_DAY).let {
                            if (it < 0) it + MINUTES_OF_DAY else it
                        }
                        val minutes = (time % 60).toString().padStart(2, '0')
                        val hours = (time / 60).toString().padStart(2, '0')
                        +"$hours:$minutes"
                    }
                }
            }
        }

        for (room in rooms) {
            div("calendar-row") {
                div("calendar-cell") {
                    span {
                        +room.name
                    }
                }

                for (i in 0 until minutesOfDay / CALENDAR_GRID_WIDTH) {
                    val start = i * CALENDAR_GRID_WIDTH + from
                    val end = (i + 1) * CALENDAR_GRID_WIDTH + from - 1

                    div("calendar-cell") {
                        val time = i * CALENDAR_GRID_WIDTH
                        val minutes = (time % 60).toString().padStart(2, '0')
                        val hours = (time / 60).toString().padStart(2, '0')
                        title = "$hours:$minutes"
                        attributes["data-time"] = start.toString()
                        attributes["data-room"] = room.id.toString()
                        attributes["data-day"] = day.toString()

                        val schedule = (start..end).mapNotNull { schedules[room]?.get(it) }.firstOrNull()

                        calendarCell(schedule)

                        val href = if (allowEdit) "/calendar/$day/${room.id}/$start" else null
                        a(href, classes = "calendar-link")
                    }
                }
            }
        }
    }
}

private fun DIV.renderRoomToTime(
    day: Int,
    from: Int,
    to: Int,
    rooms: List<Room>,
    schedules: Map<Room, Map<Int, Schedule>>,
    allowEdit: Boolean
) {
    val gridLabelWidth = 60
    val minutesOfDay = to - from

    div("calendar-table-room-to-time") {
        div("calendar-header") {
            div("calendar-cell") {
                span {
                    +"Room"
                }
            }

            for (room in rooms) {
                div("calendar-cell") {
                    span {
                        +room.name
                    }
                }
            }
        }

        for (i in 0 until minutesOfDay / CALENDAR_GRID_WIDTH) {
            val time = ((i * CALENDAR_GRID_WIDTH + from) % MINUTES_OF_DAY).let {
                if (it < 0) it + MINUTES_OF_DAY else it
            }
            val minutes = (time % 60).toString().padStart(2, '0')
            val hours = (time / 60).toString().padStart(2, '0')
            val timeString = "$hours:$minutes"

            val start = i * CALENDAR_GRID_WIDTH + from
            val end = (i + 1) * CALENDAR_GRID_WIDTH + from - 1

            div("calendar-row") {
                div("calendar-cell") {
                    if (time % gridLabelWidth == 0) {
                        span {
                            +timeString
                        }
                    }
                }

                for (room in rooms) {
                    div("calendar-cell") {
                        attributes["data-time"] = start.toString()
                        attributes["data-room"] = room.id.toString()
                        attributes["data-day"] = day.toString()
                        title = timeString

                        val schedule = (start..end).mapNotNull { schedules[room]?.get(it) }.firstOrNull()

                        calendarCell(schedule)

                        val href = if (allowEdit) "/calendar/$day/${room.id}/$start" else null
                        a(href, classes = "calendar-link")
                    }
                }
            }
        }
    }
}

fun Route.calendar() {

    get("/calendar") {
        call.respondRedirect("/calendar/0", true)
    }

    get("/calendar/{day}/rtt") {
        call.response.cookies.append(
            "orientation",
            CalendarOrientation.ROOM_TO_TIME.name,
            maxAge = Int.MAX_VALUE,
            path = "/"
        )
        val day = call.parameters["day"]?.toIntOrNull() ?: 0
        call.respondRedirect("/calendar/$day")
    }

    get("/calendar/{day}/ttr") {
        call.response.cookies.append(
            "orientation",
            CalendarOrientation.TIME_TO_ROOM.name,
            maxAge = Int.MAX_VALUE,
            path = "/"
        )
        val day = call.parameters["day"]?.toIntOrNull() ?: 0
        call.respondRedirect("/calendar/$day")
    }

    get("/calendar/{day}") {
        val user = isAuthenticated(Permission.SCHEDULE)
        val editable = user != null

        val day = call.parameters["day"]?.toIntOrNull() ?: return@get

        val rooms = RoomRepository.all()

        val orientation = call.request.cookies["orientation"]?.let { name ->
            CalendarOrientation.values().find { it.name == name }
        } ?: CalendarOrientation.ROOM_TO_TIME

        val h = ScheduleRepository.getByDay(day)
        val schedules = h.groupBy { it.room }.mapValues { (_, it) ->
            it.associateBy {
                it.time
            }
        }

        var max = 0
        var min = 24 * 60
        for (s in h) {
            max = max(max, s.time + s.workGroup.length)
            min = min(min, s.time)
        }

        if (min > max) {
            val h1 = max
            max = min
            min = h1
        }

        if (editable) {
            min = min(min, 0)
            max = max(max, 24 * 60)
        }

        min = (min / 60 - 1) * 60
        max = (max / 60 + 2) * 60

        call.respondHtmlTemplate(MainTemplate()) {
            menuTemplate {
                this.user = user
                active = MenuTemplate.Tab.CALENDAR
            }
            content {
                if (rooms.isEmpty()) {
                    return@content
                }

                div("header") {
                    div("header-left") {
                        a("/calendar/${day - 1}") { +"<" }
                        span {
                            +"Day $day"
                        }
                        a("/calendar/${day + 1}") { +">" }
                    }
                    div("header-right") {
                        a("/calendar/$day/rtt") {
                            +"Room to time"
                        }
                        a("/calendar/$day/ttr") {
                            +"Time to room"
                        }
                    }
                }

                div("calendar") {
                    attributes["data-day"] = day.toString()
                    attributes["data-editable"] = editable.toString()

                    div("calendar-table") {
                        when (orientation) {
                            CalendarOrientation.ROOM_TO_TIME -> renderRoomToTime(
                                day,
                                min,
                                max,
                                rooms,
                                schedules,
                                editable
                            )
                            CalendarOrientation.TIME_TO_ROOM -> renderTimeToRoom(
                                day,
                                min,
                                max,
                                rooms,
                                schedules,
                                editable
                            )
                        }
                    }

                    if (editable) {
                        div("calendar-edit") {
                            div("calendar-edit-top") {
                                button(classes = "form-btn") {
                                    +"Edit"
                                }
                            }
                            div("calendar-edit-main") {
                                div("calendar-edit-search") {
                                    input(InputType.search, name = "search", classes = "form-control") {
                                        placeholder = "Search"
                                        value = ""
                                    }
                                }
                                div("calendar-edit-list") {

                                }
                            }
                        }
                    }
                }
            }
        }
    }

    get("/calendar/{day}/{room}/{time}") {
        authenticateOrRedirect(Permission.SCHEDULE) { user ->
            val day = call.parameters["day"]?.toIntOrNull() ?: return@get
            val time = call.parameters["time"]?.toIntOrNull() ?: return@get
            val roomId = call.parameters["room"]?.toLongOrNull() ?: return@get
            val search = call.parameters["search"] ?: ""

            val list = WorkGroupRepository.all()
            val room = RoomRepository.get(roomId) ?: return@get

            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.WORK_GROUP
                }
                content {
                    h1 { +"Select work groups" }
                    insert(TableTemplate()) {
                        searchValue = search

                        action {
                            a("/calendar/$day") {
                                button(classes = "form-btn btn-primary") {
                                    +"Cancel"
                                }
                            }
                        }

                        header {
                            th {
                                +"Name"
                            }
                            th {
                                +"Interested"
                            }
                            th {
                                +"Track"
                            }
                            th {
                                +"Projector"
                            }
                            th {
                                +"Resolution"
                            }
                            th {
                                +"Length"
                            }
                        }

                        for (u in list) {
                            val s = u.createSearch()
                            if (Search.match(search, s)) {
                                val href = "/calendar/$day/${room.id}/$time/${u.id}"
                                entry {
                                    attributes["data-search"] = s.stringify()
                                    td {
                                        a(href) {
                                            +u.name
                                        }
                                    }
                                    td {
                                        +u.interested.toString()
                                    }
                                    td {
                                        +(u.track?.name ?: "")
                                    }
                                    td {
                                        +u.projector.toString()
                                    }
                                    td {
                                        +u.resolution.toString()
                                    }
                                    td {
                                        +u.length.toString()
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    get("/calendar/{day}/{room}/{time}/{workgroup}") {
        authenticateOrRedirect(Permission.SCHEDULE) { user ->
            val day = call.parameters["day"]?.toIntOrNull() ?: return@get
            val time = call.parameters["time"]?.toIntOrNull() ?: return@get
            val roomId = call.parameters["room"]?.toLongOrNull() ?: return@get
            val workGroupId = call.parameters["workgroup"]?.toLongOrNull() ?: return@get

            val room = RoomRepository.get(roomId) ?: return@get
            val workGroup = WorkGroupRepository.get(workGroupId) ?: return@get

            val schedule = Schedule(null, workGroup, room, day, time)
            ScheduleRepository.create(schedule)

            val redirect = call.parameters["redirect"]
            call.respondRedirect(redirect ?: "/calendar/$day")
        }
    }

    get("/calendar/{day}/{schedule}/delete") {
        authenticateOrRedirect(Permission.SCHEDULE) { user ->
            val day = call.parameters["day"]?.toIntOrNull() ?: return@get
            val scheduleId = call.parameters["schedule"]?.toLongOrNull() ?: return@get

            ScheduleRepository.delete(scheduleId)

            val redirect = call.parameters["redirect"]
            call.respondRedirect(redirect ?: "/calendar/$day")
        }
    }
}

enum class CalendarOrientation {
    /**
     * Columns contains time
     * Rows contains rooms
     *
     * Like the old kif tool
     */
    TIME_TO_ROOM,

    /**
     * Columns contains rooms
     * Rows contains time
     *
     * Like the congress schedule
     */
    ROOM_TO_TIME
}
